<?php
/**
 * Mirasvit
 *
 * This source file is subject to the Mirasvit Software License, which is available at https://mirasvit.com/license/.
 * Do not edit or add to this file if you wish to upgrade the to newer versions in the future.
 * If you wish to customize this module for your needs.
 * Please refer to http://www.magentocommerce.com for more information.
 *
 * @category  Mirasvit
 * @package   mirasvit/module-core
 * @version   1.2.63
 * @copyright Copyright (C) 2018 Mirasvit (https://mirasvit.com/)
 */



namespace Mirasvit\Core\Console\Command;

use Magento\Framework\Filesystem\DirectoryList;
use Magento\Setup\Module\I18n\Dictionary\Options\ResolverFactory;
use Magento\Setup\Module\I18n\Dictionary\WriterInterface;
use Magento\Setup\Module\I18n\Factory;
use Magento\Setup\Module\I18n\FilesCollector;
use Magento\Setup\Module\I18n\Parser;
use Magento\Setup\Module\I18n\Parser\Adapter\Php\Tokenizer\PhraseCollector;
use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Input\InputArgument;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;

class I18nFrontCollectPhrasesCommand extends Command
{
    const INPUT_KEY_MODULE = 'module';

    public function __construct(
        DirectoryList $directoryList,
        Factory $factory,
        ResolverFactory $optionResolverFactory,
        PhraseCollector $phraseCollector
    ) {
        $this->directoryList = $directoryList;
        $this->factory = $factory;
        $this->optionResolverFactory = $optionResolverFactory;
        $this->phraseCollector = $phraseCollector;

        parent::__construct();

        $this->addAdapters();
    }

    /**
     * @var Parser\Parser
     */
    protected $parser;

    /**
     * @var WriterInterface
     */
    protected $writer;

    /**
     * {@inheritdoc}
     */
    protected function configure()
    {
        $this->setName('mirasvit:collect:phrases')
            ->setDescription('Collect frontend phrases')
            ->setDefinition($this->getInputList())
        ;

        parent::configure();
    }

    /**
     * {@inheritdoc}
     */
    protected function execute(InputInterface $input, OutputInterface $output)
    {
        $withContext = false;
        $module = $input->getArgument(self::INPUT_KEY_MODULE);
        $rootPath = $this->directoryList->getRoot();
        $directory = $rootPath . '/vendor/mirasvit/module-' . lcfirst($module) . '/src/' . ucfirst($module) . '/';
        $outputFilename = $directory . 'i18n/front_en_US.csv';
        $optionResolver = $this->optionResolverFactory->create($directory, $withContext);

        $options = $optionResolver->getOptions();
        foreach ($options as $k => $option) { // exclude backend files
            $options[$k]['fileMask'] = '/^(?:(?!\b[Aa]dminhtml\b).)*'.ltrim($options[$k]['fileMask'], '/');
        }

        $phraseList = $this->parser->parse($options);
        if (!count($phraseList)) {
            throw new \UnexpectedValueException('No phrases found in the specified dictionary file.');
        }
        foreach ($phraseList as $phrase) {
            $this->getDictionaryWriter($outputFilename)->write($phrase);
        }
        $this->writer = null;
    }

    /**
     * @return void
     */
    private function addAdapters()
    {
        $filesCollector = new FilesCollector();
        $phraseCollector = new Parser\Adapter\Php\Tokenizer\PhraseCollector(new Parser\Adapter\Php\Tokenizer());
        $adapters = [
            'php' => new Parser\Adapter\Php($phraseCollector),
            'html' => new Parser\Adapter\Html(),
            'js' => new Parser\Adapter\Js(),
            'xml' => new Parser\Adapter\Xml(),
        ];

        $factory = new \Magento\Setup\Module\I18n\Factory();
        $parser = new Parser\Parser($filesCollector, $factory);
        foreach ($adapters as $type => $adapter) {
            $parser->addAdapter($type, $adapter);
        }

        $this->parser = $parser;
    }

    /**
     * @param string $outputFilename
     * @return WriterInterface
     */
    protected function getDictionaryWriter($outputFilename)
    {
        if (null === $this->writer) {
            $this->writer = $this->factory->createDictionaryWriter($outputFilename);
        }
        return $this->writer;
    }

    /**
     * Command definition.
     *
     * @return array
     */
    private function getInputList()
    {
        return [
            new InputArgument(
                self::INPUT_KEY_MODULE,
                InputArgument::REQUIRED,
                'Module name.'
            ),
        ];
    }
}
